/***************************************************************************
 *   Copyright (C) 2003 by Sbastien Laot                                 *
 *   slaout@linux62.org                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef SEARCH_H
#define SEARCH_H

#include <qwidget.h>
#include <qmap.h>
//#include <qtimer.h>

class KLineEdit;
class QComboBox;
//class KToolBarButton;
class QToolButton;

class Tag;
class State;

/** The structure that contain all search terms
  * @author Sbastien Laot
  */
class SearchData
{
  public:
	// Useful Enum for tagSearchType:
	enum TagSearchType { DontCareTagsSearch = 0, NotTaggedSearch, TaggedSearch, TagSearch, StateSearch };
	// Constructor and Destructor:
	SearchData()  { isSearching = false; tagSearchType = DontCareTagsSearch; tag = 0; state = 0; }
	~SearchData() {}
	// Search data:
	QString  string;
	int      tagSearchType;
	Tag     *tag;
	State   *state;
	bool     isSearching; // FIXME: Useful?
};

/** A QWidget that allow user to enter terms to search in a Basket.
  * @author Sbastien Laot
  */
class SearchBar : public QWidget
{
  Q_OBJECT
  public:
	SearchBar(QWidget *parent = 0, const char *name = 0);
	~SearchBar();
	const SearchData& searchData();
  signals:
	void newSearch(const SearchData &data);
  public slots:
	void repopulateTagsComnbo();
	void reset();
	void inAllBaskets();
	void setEditFocus();
	void searchTag(Tag *tag);
	void searchState(State *state);
	void setSearchAll(bool searchAll);
	void setSearchData(const SearchData &data);
  public:
	bool hasEditFocus();
//	void show();
//	void hide();
	KLineEdit* lineEdit() { return m_lineEdit; }
  private slots:
	void textChanged(const QString &text);
	void tagChanged(int index);
//	void blinkBar();
  private:
	SearchData     *m_data;
	KLineEdit      *m_lineEdit;
//	KToolBarButton *m_resetButton; // FIXME: TODO: Make the bar a ToolBar and then plug KActions
	QToolButton    *m_resetButton;
	QComboBox      *m_tagsBox;
//	KToolBarButton *m_inAllBasketsButton;
	QToolButton    *m_inAllBasketsButton;

	QMap<int, Tag*>   m_tagsMap;
	QMap<int, State*> m_statesMap;
//	QTimer          m_blinkTimer;
//	int             m_blinkedTimes;
};

#endif // SEARCH_H
